import { Component, ViewChild } from "@angular/core";
import {
  FormBuilder,
  FormGroup,
  NgForm,
  Validators,
  AbstractControl,
  ValidatorFn,
} from "@angular/forms";
import { ActivatedRoute, Router } from "@angular/router";
import { LoginService } from "../../service/login.service";
import { ToastrService } from "ngx-toastr";
import { AuthService } from "src/app/_services/auth.service";

@Component({
  selector: "app-update-password",
  templateUrl: "./update-password.component.html",
  styleUrls: ["./update-password.component.css"],
})
export class UpdatePasswordComponent {
  @ViewChild("f", { read: true }) userProfileForm: NgForm;
  public breadcrumb: any;
  profilInfo: FormGroup;
  activeTab: string = "profile-info2";
  submitted = false;
  showCurrentPass: boolean = false;
  showNewPass: boolean = false;
  showConfirmPass: boolean = false;

  storedData: any;

  constructor(
    private formBuilder: FormBuilder,
    private router: Router,
    private route: ActivatedRoute,
    private authService: LoginService,
    private toastr: ToastrService,
    private logoutService: AuthService,
  ) {}

  ngOnInit(): void {
    this.storedData = JSON.parse(localStorage.getItem("account_info"));
    this.authService.checkTokenAndRedirect();
    this.profilInfo = this.formBuilder.group({
      userid: this.storedData.sub,
      currentPass: [""],
      newPass: [
        "",
        [
          Validators.required,
          Validators.minLength(8),
          Validators.pattern(/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d).+$/),
        ],
      ],
      confirmPass: ["", [Validators.required, this.matchPassword("newPass")]],
    });

    this.profilInfo.get("newPass").valueChanges.subscribe(() => {
      this.profilInfo.get("confirmPass").updateValueAndValidity();
    });
  }

  selectTab(tabName: string) {
    this.activeTab = tabName;
  }

  get f() {
    return this.profilInfo.controls;
  }

  matchPassword(passwordControlName: string): ValidatorFn {
    return (control: AbstractControl): { [key: string]: boolean } | null => {
      if (!control.parent || !control) {
        return null;
      }

      const passwordControl = control.parent.get(passwordControlName);
      if (!passwordControl) {
        return null;
      }

      if (passwordControl.value !== control.value) {
        return { matchPassword: true };
      }

      return null;
    };
  }

  onProjectInfoSubmit() {
    this.submitted = true;

    if (this.profilInfo.invalid) {
      return;
    }
  }

  cancel() {
    this.profilInfo = this.formBuilder.group({
      userid: this.storedData.sub,
      currentPass: [""],
      newPass: [
        "",
        [
          Validators.required,
          Validators.minLength(8),
          Validators.pattern(/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d).+$/),
        ],
      ],
      confirmPass: ["", [Validators.required, this.matchPassword("newPass")]],
    });
  }

  saveEdit() {
    this.submitted = true;
    if (this.profilInfo.valid) {
      const dataPassword = {
        userid: this.profilInfo.value.userid,
        password: this.profilInfo.value.newPass,
      };
      this.authService.updatePassword(dataPassword).subscribe(
        (res) => {
          this.toastr.success("Success", "Update Password Completed.", {
            timeOut: 2000,
            closeButton: true,
          });
          this.logout()
        },
        (error) => {
          console.error(error);
          this.toastr.error("Error", "Something went wrong!", {
            timeOut: 2000,
            closeButton: true,
          });
        }
      );
    } else {

    }
  }

  logout() {
    if (localStorage.getItem('currentUser')) {
      this.logoutService.doLogout().then(res => {
        window.location.href = '/login';
      }, err => {
        console.log(err);
      });
    }
  }

  toggleCurrentPassVisibility() {
    this.showCurrentPass = !this.showCurrentPass;
  }

  toggleNewPassVisibility() {
    this.showNewPass = !this.showNewPass;
  }

  toggleConfirmPassVisibility() {
    this.showConfirmPass = !this.showConfirmPass;
  }
}
